#RSDeltaSigmaPort: Implement missing core functionality
#-------------------------------------------------------------------------------

"""Emulates rounding of array indices"""
array_round(i) = round(Int, i) #VERIFYME

#Sort complex poles
function _cplxpair(a)
	epstol = 100

	#Sorts real first, then imaginary:
	function isrealless(a,b)
		Δmin = epstol*eps(abs(a))
		if abs(real(a) - real(b)) < Δmin
			return imag(a) < imag(b)
		else
			return real(a) < real(b)
		end
	end

	a = sort(a, lt=isrealless)
	i = 1
	while i < length(a)
		v = a[i]
		Δmin = epstol*eps(abs(v))
		if abs(imag(v)) < Δmin #Really a real number
			a[i] = real(v)
			i+=1
			break
		elseif i < length(a)
			if abs(v-conj(a[i+1])) < Δmin
				i+=2 #Skip both cplx conj pairs.
				break
			else
				throw(ErrorException("cplxpair failed."))
			end
		else
			throw(ErrorException("cplxpair failed."))
		end
		
	end
	return a
end

"""`cplxpair(a)`

Sort roots in array a such that complex pairs are in ascending order of their
real parts, then by their imaginary part.  In the output, the real roots
re-positionned after the complex roots.
"""
function cplxpair(a)
	epstol = 100

	#Index of real roots:
	Δmin = epstol*eps.(abs.(a)) #Threshold for numerical error
	ireal = abs.(imag.(a)) .< Δmin

	areal = real.(a[ireal]) #Clean up real roots
	acplx = a[.!ireal] #complex pairs
	return vcat(_cplxpair(acplx), sort(areal)) #Sort roots
end

#Custom function to ensure complex conjugate pairs match exactly.
function cleancplxpair(l)
	epstol = 100

	if length(l) < 2
		return l
	end

	l = cplxpair(l) #Sort to group complex conjugates together
	result = similar(l)
	if length(l)>0
		#Copy last value in case not part of conjugate pair
		#(ignored by following algorithm)
		result[end] = l[end]
	end

	i = 1
	while i < length(l)
		a, b = l[i], l[i+1]
		ΔRmin = epstol*eps(real(a))
		ΔXmin = epstol*eps(imag(a))

		result[i] = a
		if abs(real(a) - real(b)) < ΔRmin && abs(imag(a) + imag(b)) < ΔXmin
			a = (a+conj(b))/2 #Take mean & re-write
			result[i] = a
			result[i+1] = conj(a)
			i+=1
		end
		i+=1
	end

	return result
end

#TODO: Will algorithms work without collect (possibly faster?):
eye(n::Int) = collect(LinearAlgebra.I(n))

#It sounds like this should be implementation of orth:
#orth(A) = LinearAlgebra.qr(A).Q

#But scipy seems to do it this way:
orth(A) = LinearAlgebra.svd(A).U

#Convert array of roots into polynomial:
function poly(rA)
	result = [1]
	for r in rA
		result = conv(result, [1, r])
	end
	return result
end

eig(a) = eigen(a).values

#Implement interp1 using linear interpolation:
function interp1_lin(xA, yA, xv)
	ILib = Interpolations
	itp = ILib.interpolate((x,), y, ILib.Gridded(ILib.Linear()))
		return itp(xv)
end

#Implement interp1 using cubic interpolation:
function interp1_cubic(xA, yA, xv)
	ILib = Interpolations
   itp = ILib.interpolate(yA, ILib.BSpline(ILib.Cubic(ILib.Natural())), ILib.OnGrid())
		intf = ILib.scale(itp, xA)
	return intf(xv)
end
#Last line
